/**
 * File:   cObject.h
 * Author: Benjamin Pickhardt (hackCraft.de)
 *
 * Created on March 28, 2008, 10:23 PM
 *
 * Generic Game-Object with as much common
 * attributes as are making sense and needed
 * for various World-Filtering (Worldsense for Objects).
 */

#ifndef OBJECT_INC
#define OBJECT_INC

class cObject;
class cPad;
class cController;

#include "cWorld.h"
#include <string>

using namespace std;

/**
 * Generic Game-Object (Object with Roles) which
 * collects as much common attributes as are needed
 * by the different roles and as needed by World-Filtering
 * (Worldsense for subclasses implementing the roles).
 * This object does again holds as much methods as needed
 * but without (real) implementation (which is done by the subclasses).
 * Nevertheless every method has a empty body or a body returning
 * some default value, so that callers need not care about
 * the kind of the object (roles) too much.
 */
class cObject {
public:
  friend class cWorld;
  friend class cController;
  
  enum Roles {
    // Roles indicating Attributes
    NAMEABLE,
    SOCIAL,
    TRACKABLE,
    DAMAGEABLE,
    ENTITY,
    
    // Roles indicating Kind of Objects
    MECH,
    GEOSPOT,
    BUILDING,
    TREE,
    
    // Roles indicating Social-Groups
    RED,
    BLUE,
    GREEN,
    YELLOW,
    ARMY,
    MERCENARY,
    CIVIL,

    // Roles indicating State
    RELOADING,

    HEALTHY,
    WOUNDED,
    CRITICAL,
    DEAD,

    // Special Roles
    HUMANPLAYER,
  };
  
  // BASE
  cWorld* mWorld; // The world this object was spawned in.
  bool mFragged; // Is this object scheduled to be deleted after the next frame.
  unsigned long mRoles; // Bitmask, lists the roles this object plays.

  // NAMEABLE
  string* mName;
  string* mDescription;
  unsigned int mDesignation; // { 0, .., 25 } for { a, ..., z }.

  // SOCIAL
  unsigned long mAllies; // future: Bitmask, lists Roles which Allies have.
  unsigned long mEnemies; // Bitmask, lists Roles which Enemies play.
  
  // TRACKABLE
  float mPos[3];
  float mRot[3];
  float mRadius;
  
  enum Parts {
      LEFT,
      RIGHT,
      BODY,
      LEGS,
      MAX_PARTS
  };
  
  // DAMAGEABLE
  float mHp;
  //int* mSplasharmor;
  //int* mBeamarmor;
  //int* mPiercearmor;
  // int mHeat;
  // int mHeatsinks;
  // int mSplasharmor;
  // int mBeamarmor;
  // int mPiercearmor;

  // ENTITY
  cObject* mTarget; // Aim target
  float mDestination[3]; // Movement target
  cPad* mPad; // Direct Human and Remote Pad-Control.
  cController* mController; // Command Stack takes controll when (available and) enabled.
  
  // WEAPON

public:
  cObject();
  cObject(cObject* source);
  virtual ~cObject() {};

  virtual void onSpawn() {};
  virtual void onFrag() {};
  
  virtual void multEyeMatrix() {};
  
  virtual void animateSelf(float spf) {};
  virtual void transformSelf() {};
  virtual void drawSolid() {};
  virtual void drawEffect() {};
  virtual void drawHUD() {};
  
  // Receiving damage:
  
  //virtual void damage(float damage) {};

  struct rImpact { float pos[3]; float nrm[3]; };
  
  virtual rImpact* damageByCylinder(float* a, float* b, float radius, float damage, bool all) {};
  //virtual rImpact* damageBySphere(float* a, float radius, float damage, bool all) {};
  
  
  // Related to States:
  
  // Queries
  virtual cObject* enemyNearby() { return NULL; };
  
  // Fuzzy Predicates [0,1]
  virtual float inMeeleRange() { return 0.0f; };
  virtual float inWeaponRange() { return 0.0f; };
  virtual float inTargetRange() { return 0.0f; };

  // Output Actions for Entities
  virtual void do_moveTowards() {};
  virtual void do_moveNear() {};
  virtual void do_aimAt() {};
  virtual void do_fireAt() {};
  virtual void do_idle() {};
  virtual void do_aimFor(cObject* target) {};
};

#endif
